//use clap::Parser;
use clap::builder::{Arg, Command, NonEmptyStringValueParser, ValueHint};

const HOST_HELP: &str = "Host(s) the action should apply to (regex)";
const SERVICE_HELP: &str = "Service(s) the action should apply to (regex)";
const HOSTER_HELP: &str = "Hoster(s) the action should apply to (regex)";

#[allow(clippy::too_many_lines)]
#[must_use]
pub fn build() -> Command {
    //    const TARGET_HELP: &str = "Target of the action, format: <host>[!<service>[!<hoster>]] (regex)";
    Command::new("i2cmd")
        .version("Command line utility for icinga")
        .author("Mathieu Trossevin <mtrossevin@evolix.fr>")
        .about("Command line utility for icinga")
        .subcommand_required(true)
        .arg(
            Arg::new("config")
                .short('c')
                .long("config")
                .value_name("CONF_FILE")
                .help("Sets a custom config file")
                .num_args(1)
                .value_parser(clap::builder::PathBufValueParser::new())
                .value_hint(ValueHint::FilePath)
                .default_value("conf.toml"),
        )
        .arg(
            Arg::new("author")
                .short('a')
                .long("author")
                .value_name("AUTHOR")
                .help("Sets the author of actions in icinga [env: USER if not configured in CONF_FILE]")
                .long_help("Sets the author of actions in icinga (simply an info for other users)\n[env: USER if not configured in CONF_FILE]")
                .num_args(1)
                .value_parser(NonEmptyStringValueParser::new())
                .value_hint(ValueHint::Username),
        )
        .subcommand(
            clap::Command::new("ack")
                .disable_version_flag(true)
                .about("Acknowledge a problem")
                .arg(Arg::new("service")
                    .short('s')
                    .long("service")
                    .value_name("SERVICE")
                    .num_args(1)
                    .help(SERVICE_HELP)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other),
                )
                .arg(Arg::new("hoster")
                    .short('H')
                    .long("hoster")
                    .value_name("HOSTER")
                    .num_args(1)
                    .help(HOSTER_HELP)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other),
                )
                .arg(Arg::new("sticky")
                    .long("sticky")
                    .help("The acknowledgement will remain until full recovery")
                    .action(clap::ArgAction::SetTrue)
                )
                .arg(
                    Arg::new("host")
                        .index(1)
                        .required(true)
                        .value_name("host")
                        .help(HOST_HELP)
                        .num_args(1)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Hostname)
                )
                .arg(
                    Arg::new("comment")
                        .index(2)
                        .required(true)
                        .value_name("COMMENT")
                        .help("Comment")
                        .num_args(1..)
                        .trailing_var_arg(true)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Other)
                )
        )
        .subcommand(
            clap::Command::new("ackexp")
                .disable_version_flag(true)
                .about("Acknowledge a problem for some time")
                .arg(Arg::new("service")
                    .short('s')
                    .long("service")
                    .value_name("SERVICE")
                    .num_args(1)
                    .help(SERVICE_HELP)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                )
                .arg(Arg::new("hoster")
                    .short('H')
                    .long("hoster")
                    .value_name("HOSTER")
                    .num_args(1)
                    .help(HOSTER_HELP)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                )
                .arg(Arg::new("sticky")
                    .long("sticky")
                    .help("The acknowledgement will remain until full recovery or expiry")
                    .action(clap::ArgAction::SetTrue)
                )
                .arg(
                    Arg::new("host")
                        .index(1)
                        .required(true)
                        .value_name("host")
                        .help(HOST_HELP)
                        .num_args(1)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Hostname)
                )
                .arg(
                    Arg::new("end_time")
                        .index(2)
                        .required(true)
                        .num_args(1)
                        .value_name("END_TIME")
                        .help("End of acknowledgement, format: X[w|d|h|m|s] (repeatable, no space)")
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Other)
                )
                .arg(
                    Arg::new("comment")
                        .index(3)
                        .required(true)
                        .value_name("COMMENT")
                        .help("Comment")
                        .num_args(1..)
                        .trailing_var_arg(true)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Other)
                )
        )
        .subcommand(
            clap::Command::new("com")
                .disable_version_flag(true)
                .about("Add a comment on the TARGET")
                .arg(Arg::new("service")
                    .short('s')
                    .long("service")
                    .value_name("SERVICE")
                    .num_args(1)
                    .help(SERVICE_HELP)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                )
                .arg(Arg::new("hoster")
                    .short('H')
                    .long("hoster")
                    .value_name("HOSTER")
                    .num_args(1)
                    .help(HOSTER_HELP)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                )
                .arg(
                    Arg::new("host")
                        .index(1)
                        .required(true)
                        .value_name("host")
                        .help(HOST_HELP)
                        .num_args(1)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Hostname)
                )
                .arg(
                    Arg::new("comment")
                        .index(2)
                        .required(true)
                        .value_name("COMMENT")
                        .help("Comment")
                        .num_args(1..)
                        .trailing_var_arg(true)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Other)
                )
        )
        .subcommand(
            clap::Command::new("dt")
                .disable_version_flag(true)
                .about("Schedule a downtime")
                .arg(Arg::new("service")
                    .short('s')
                    .long("service")
                    .value_name("SERVICE")
                    .num_args(1)
                    .help(SERVICE_HELP)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                )
                .arg(Arg::new("hoster")
                    .short('H')
                    .long("hoster")
                    .value_name("HOSTER")
                    .num_args(1)
                    .value_parser(NonEmptyStringValueParser::new())
                    .help(HOSTER_HELP)
                    .value_hint(ValueHint::Other)
                )
                .arg(Arg::new("flexible")
                    .long("flexible")
                    .value_name("DURATION")
                    .num_args(1)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                    .help("Make a flexible downtime")
                )
                .arg(Arg::new("start_time")
                    .long("start")
                    .value_name("TIME")
                    .num_args(1)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                    .help("Start of downtime (RFC 3339)")
                )
                .arg(
                    Arg::new("host")
                        .index(1)
                        .required(true)
                        .value_name("host")
                        .help(HOST_HELP)
                        .num_args(1)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Hostname)
                )
                .arg(
                    Arg::new("end_time")
                        .index(2)
                        .required(true)
                        .num_args(1)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_name("END_TIME")
                        .help("End of downtime, format: X[w|d|h|m] (repeatable, no space)")
                        .value_hint(ValueHint::Other)
                )
                .arg(
                    Arg::new("comment")
                        .index(3)
                        .required(true)
                        .value_name("COMMENT")
                        .help("Comment")
                        .num_args(1..)
                        .trailing_var_arg(true)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Other)
                )
        )
        .subcommand(
            clap::Command::new("rm")
                .disable_version_flag(true)
                .about("Remove all acknowledgements and downtime on the TARGET")
                .arg(Arg::new("service")
                    .short('s')
                    .long("service")
                    .value_name("SERVICE")
                    .num_args(1)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                    .help(SERVICE_HELP)
                )
                .arg(Arg::new("hoster")
                    .short('H')
                    .long("hoster")
                    .value_name("HOSTER")
                    .num_args(1)
                    .value_parser(NonEmptyStringValueParser::new())
                    .value_hint(ValueHint::Other)
                    .help(HOSTER_HELP)
                )
                .arg(
                    Arg::new("host")
                        .index(1)
                        .required(true)
                        .value_name("host")
                        .help(HOST_HELP)
                        .num_args(1)
                        .value_parser(NonEmptyStringValueParser::new())
                        .value_hint(ValueHint::Hostname)
                )
        )
}

// #[derive(Debug, Parser)]
// #[command(author = "Mathieu Trossevin <mtrossevin@evolix.fr>", version)]
// /// Command line utility for icinga
// pub struct I2Cmd {
//     #[arg(short, long, default_value = "conf.toml")]
//     /// Sets a custom config file
//     config: std::path::PathBuf,
//     #[arg(short, long, value_hint = ValueHint::Username)]
//     /// Sets the author of actions in icinga [env: USER if not configured in CONF_FILE]
//     ///
//     /// This is simply an info for other users.
//     author: Option<String>,
//     #[command(subcommand)]
//     action: Action,
// }

// #[derive(Parser, Debug)]
// pub enum Action {
//     /// Acknowledge a problem
//     Ack(Ack),
// }

// #[derive(Debug, Parser)]
// pub struct Ack {
//     #[arg(short, long, help = SERVICE_HELP, value_parser = NonEmptyStringValueParser::new(), value_hint = ValueHint::Other)]
//     service: Option<String>,
//     #[arg(short = 'H', long, help = HOSTER_HELP, value_parser = NonEmptyStringValueParser::new(), value_hint = ValueHint::Other)]
//     hoster: Option<String>,
//     #[arg(long)]
//     sticky: bool,
//     #[arg(index = 1, help = HOST_HELP, value_parser = NonEmptyStringValueParser::new(), value_hint = ValueHint::Hostname)]
//     host: String,
//     #[arg(index = 2, required = true, trailing_var_arg = true, num_args = 1.., value_parser = NonEmptyStringValueParser::new(), value_hint = ValueHint::Other)]
//     /// Comment
//     comment: Vec<String>
// }
